﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace AnotherWobulator
{
    public partial class Main : Form
    {
        //**************************************************************************************************
        private List<string> lsTotalEntries = new List<string>();
        private List<string> lsEligibleEntries = new List<string>();
        private List<string> lsDuplicatedEntries = new List<string>();
        private List<string> lsBadUsers = new List<string>();
        private List<string> lsSmallerPoster = new List<string>();
        private Regex rxEntries = new Regex(@"(?isx)post_wrapper.*?""post"".*?</div>", RegexOptions.Compiled);
        private Regex rxUser = new Regex(@"(?isx)View\sthe\sprofile\sof.*?>(?<username>.*?)<", RegexOptions.Compiled);
        private Regex rxPosts = new Regex(@"(?isx)Posts:\s(?<posts>\d*)", RegexOptions.Compiled);
        private int RandomSeed = 0;
        
        //**************************************************************************************************
        public Main()
        {
            InitializeComponent();
        }

        //**************************************************************************************************
        private void Main_Load(object sender, EventArgs e)
        {
#if DEBUG
            textBoxURL.Text = "http://www.eevblog.com/forum/contests/giveaway-altium-circuitstudio-licenses";
#endif
        }

        //**************************************************************************************************
        private void buttonCollect_Click(object sender, EventArgs e)
        {
            try
            {
                if (textBoxURL.Text.Trim() != "" && !textBoxURL.Text.Trim().EndsWith("/?all"))
                    textBoxURL.Text += "/?all";

                //reset numbers
                RandomSeed = 0;
                lsTotalEntries.Clear();
                lsEligibleEntries.Clear();
                lsDuplicatedEntries.Clear();
                lsBadUsers.Clear();
                lsSmallerPoster.Clear();

                //reset UI elements
                labelTotalEntries.Text = "";
                labelDuplicatedEntries.Text = "";
                labelBadUsers.Text = "";
                labelUsersWithFewPosts.Text = "";
                labelEligibleUsers.Text = "";
                label1st.Text = "";
                label2st.Text = "";
                label3st.Text = "";
                label4st.Text = "";
                label5st.Text = "";
                this.Refresh();

                //lets see what we get...
                using (WebClient client = new WebClient())
                {
                    string content = client.DownloadString(textBoxURL.Text);
                    //use content lenght as the "non-machine" element of the random seed (better than nothing)
                    RandomSeed = content.Length;
                    
                    MatchCollection mc = rxEntries.Matches(content);

                    bool RemovedFirst = false;
                    foreach (Match m in mc)
                    {
                        //remove the first post of the thread
                        //that would be the guy making the contest
                        if (checkRemoveFirstPoster.Checked && !RemovedFirst)
                        {
                            RemovedFirst = true;
                            continue;
                        }

                        //get entry relevent values
                        string username = rxUser.Match(m.Value).Groups["username"].Value;
                        string howmanyposts = rxPosts.Match(m.Value).Groups["posts"].Value;

                        //is it a duplicated entry?
                        if (lsTotalEntries.Contains(username))
                        {
                            lsDuplicatedEntries.Add(username);
                            if (!lsBadUsers.Contains(username))
                                lsBadUsers.Add(username);
                        }
                        
                        lsTotalEntries.Add(username);
                        
                        int nPosts = 0;
                        if (Int32.TryParse(howmanyposts, out nPosts))
                        {
                            if (nPosts < numericUpDown1.Value)
                                lsSmallerPoster.Add(username);
                        }
                    }
                }

                labelTotalEntries.Text = lsTotalEntries.Count.ToString();

                if (checkRemoveDuplicateEntries.Checked)
                    labelDuplicatedEntries.Text = lsDuplicatedEntries.Count.ToString();
                else
                    labelDuplicatedEntries.Text = "---";

                if (checkRemoveBadUsers.Checked)
                    labelBadUsers.Text = lsBadUsers.Count.ToString();
                else
                    labelBadUsers.Text = "---";

                if (checkRemoveSmallerPosters.Checked)
                    labelUsersWithFewPosts.Text = lsSmallerPoster.Count.ToString();
                else
                    labelUsersWithFewPosts.Text = "---";

                //calculate final eligible list
                foreach (string user in lsTotalEntries)
                {
                    if (!checkRemoveDuplicateEntries.Checked || !lsEligibleEntries.Contains(user))
                        lsEligibleEntries.Add(user);
                }

                if (checkRemoveBadUsers.Checked)
                {
                    foreach (string user in lsBadUsers)
                    {
                        while(lsEligibleEntries.Remove(user));
                    }
                }

                if (checkRemoveSmallerPosters.Checked)
                {
                    foreach (string user in lsSmallerPoster)
                    {
                        while(lsEligibleEntries.Remove(user));
                    }
                }
                
                labelEligibleUsers.Text = lsEligibleEntries.Count.ToString();
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        //**************************************************************************************************
        private void buttonDraw_Click(object sender, EventArgs e)
        {
            if (lsEligibleEntries.Count == 0)
            {
                MessageBox.Show("Draw?!? From a pool of 0?");
                return;
            } 
            
            //just change the seed a litle before draw
            RandomSeed += DateTime.Now.Millisecond;
            Random rand = new Random(RandomSeed);
            int winnerIndex = rand.Next(0, lsEligibleEntries.Count - 1);
            string username = lsEligibleEntries[winnerIndex];

            switch (((Button)sender).Name)
            {
                case "button1st":
                    label1st.Text = username;
                    break;
                case "button2st":
                    label2st.Text = username;
                    break;
                case "button3st":
                    label3st.Text = username;
                    break;
                case "button4st":
                    label4st.Text = username;
                    break;
                case "button5st":
                    label5st.Text = username;
                    break;
            }

            while(lsEligibleEntries.Remove(username)); 
        }
    }
}
